import java.util.*;
/**
 * this class represents a rule set.
 * a rule set is a set that contains all the PPMVs that hold a specific rule
 * a rule is 2 suffixes and a ppmv is one stem that has those 2 suffixes
 * @author Danny Shacham
 */
public class RuleSet {
	private Rule rule;
	private Vector ppmvs; 
	
	/**
	 * constructs a new ruleSet that holds a specific rule
	 * @param rule1
	 * @param rule2
	 */
	public RuleSet(Rule rule) {
		this.rule = rule;
		ppmvs = new Vector();
	}
	/**
	 * adds a PPMV to this rule set, only if this PPMV implements this rule
	 * @param next the PPMV to be added to the RuleSet
	 */
	public void addPPMV(PPMV next) {
		if (next.getRule().equals(rule)) {
			ppmvs.add(next);			
		}
	}
	/**
	 * gets all the PPMVs held by this RuleSet
	 * @return Vector all the PPMVs
	 */
	public Vector getAllPPMV() {
		return ppmvs;
	}
	/**
	 * return the rule of this RuleSet
	 * @return Rule the rule of this RuleSet
	 */
	public Rule getRule() {
		return rule;
	}
	public String toString() {
		return rule.toString();
	}

}
