import java.util.*;
/**
 * a SuffixCounter is a counter that holds a suffix and all the stems of this
 * suffix.
 * a SuffixCounter also counts the number of suffixes with different stems
 * @author Danny Shacham
 *
 */
public class SuffixCounter implements Comparable {
	
	private String suffix;
	private Vector stems;
	/**
	 * constructs a SuffixCount for the given word with a count of 1
	 * @param word the word that is being counted
	 */
	public SuffixCounter(Word word) {
		this.suffix = word.getSuffix();
		stems = new Vector();
		addSuffix(word);
	}
	/**
	 * adds 1 to the counter of this word
	 */
	public void addSuffix(Word word) {
		if (!stems.contains(word.getStem())) {
			stems.add(word.getStem());
		}
	}
	/**
	 * returns a Vector of Strings, each String is a stem of this suffixCounter
	 * @return Vector
	 */
	public Vector getStems() {
		return stems;
	}
	/**
	 * gets the counts of the word
	 * @return int the counts of the words
	 */
	public int getCounts() {
		return stems.size();
	}
	/**
	 * gets the word that is being counted
	 * @return String the word that is being counted
	 */
	public String getSuffix() {
		return suffix;
	}
	/**
	 * checks if a given Object is equals to this one if the given Object is
	 * instanceof this one and the word contained by the 2 Object is
	 * equalsIgnoreCase
	 * @param o
	 * @return boolean
	 */
	public boolean equals(Object o) {
		
		SuffixCounter temp = (SuffixCounter)o;
		return suffix.equalsIgnoreCase(temp.suffix);
	}
	public String toString() {
		return getSuffix()+"	"+getCounts();
	}
	/**
	 * compares an object which is instanceof WordCount to this object.
	 * this method compares the counts of each object regardeless of the word
	 * they hold
	 * @see java.lang.Comparable#compareTo(Object)
	 */
	public int compareTo(Object o) {
		SuffixCounter temp = (SuffixCounter)o;
		if (getCounts()>temp.getCounts()) {
			return 1;
		} else {
			if (getCounts() == temp.getCounts()) {
				return 0;
			} else {
				return -1;
			}
		}		
	}
	
}
