import java.util.*;
/**
 * @author Danny Shacham
 * a class that represents a data structure of Trie which is like a tree
 * holding Strings, each node holds a char and the branches  each branch
 * represents a word in this database.
 * a word can also be a part of a branch
 */
public class Trie {
	
	private String stem;
	private Vector allWords;
	private Character letter;
	private Vector branch;
	private boolean isWord;
	
	/**
	 * constructs an empty Trie
	 * @see java.lang.Object#Object()
	 */
	public Trie() {
		this(null);		
	}
	
	/**
	 * returns the branches of this Trie
	 * @return Vector which is all the Trie's branches
	 * each element in the Vector is a Trie itself
	 */
	public Vector getBranches() {
		return branch;
	}
	/**
	 * returns the char of this Trie
	 * the root holds the char '\0'
	 * @return char the char of this Trie
	 */
	public char getCahr() {
		return letter.charValue();
	}
	
	/**
	 * puts a word into this trie
	 * @param word the word to be put
	 * @return Trie the Trie after the insertion
	 */
	public void put(String word) {		
		Character[] chars = StringToArray(word);
		put(chars);				
	}
	/**
	 * checks if a given word is in this Trie
	 * @param word the word to be checked
	 * @return boolean true if the word was inserted to this Trie
	 */
	public boolean exists(String word) {
		Character[] chars = StringToArray(word);
		return exists(chars);
	}
	/**
	 * returns a vector containg all the words in this Trie
	 * @return Vector that holds all the words in this Trie
	 */
	public Vector getSuffixes() {
		return allWords;
	}
	public String getStem() {
		return stem;
	}
	/******************   private methods     ****************************/
	// a private constructor
	private Trie(Character c) {
		letter = c;
		stem = "";
		branch = new Vector();
		isWord = false;
		allWords = new Vector();
	}
	private boolean exists(Character[] chars) {
		if (chars.length == 0) {
			return isWord;
		}
		int index = indexOf(chars[0]);
		if (index == -1) {
			return false;
		}
		Trie t = (Trie)branch.elementAt(index);
		return	t.exists(subArray(chars,1));		
	}
	private String CharacterToString(Character[] chars) {
		StringBuffer sb = new StringBuffer();
		for(int i=0; i< chars.length-1; i++) {
			sb.append(chars[i].charValue());
		}
		return sb.toString();
	}
	private void put(Character[] chars) {		
		Trie t = new Trie();
		addWordToVector(chars);
		if (chars.length == 1) {
			t.isWord = true;
			t.letter = chars[0];
			branch.add(t);
			return;
		}		
		int index = indexOf(chars[0]);	
		if (index == -1) {
			addWord(chars);
			return;
		}			
		t = (Trie)branch.elementAt(index);
		t.put(subArray(chars,1));
			
	}
	private Character[] subArray(Character[] chars, int from) {
		int size = chars.length-from;
		Character[] c = new Character[size];
		for(int i=0; i<size; i++) {
			c[i] = chars[from+i];
		}
		return c;
	}
	private void addWordToVector(Character[] chars) {
		String suffix = CharacterToString(chars);
		Word word = new Word(stem+letter, suffix);
		if (!allWords.contains(word)) {
			allWords.add(word);
		}
	}	
	private void addWord(Character[] chars) {
		addWordToVector(chars);
		if (chars.length == 0) {
			return;
		}			
		Trie t = new Trie(chars[0]);
		if (chars.length == 1) {
			t.isWord = true;
		}
		branch.add(t);
		if (letter != null) {
			t.stem = stem+letter;
		} else {
			t.stem = stem;
		}
		t.addWord(subArray(chars,1));				
	}
	private int indexOf(Character c) {
		for (int i=0; i<branch.size(); i++) {
			if (((Trie)branch.elementAt(i)).letter.equals(c)) {
				return i;
			}
		}
		return -1;
	} 
	private Character[] StringToArray(String word) {
		StringBuffer sb = new StringBuffer(word);
		Character[] c = new Character[word.length()+1];
		for (int i=0; i<c.length-1; i++) {
			c[i] = new Character(sb.charAt(i));
		}
		c[c.length-1] = new Character('*');
		return c;
	} 
		

}
